;; EasyPIM: The simple emacs tool to store personal address book
;; Michael Pozhidaev <msp@altlinux.org>
;; Date: 2011-03-23

(defcustom easypim-mail-chars "a-zA-Z0-9@._\\-" "The set of characters to be used as e-mail address in PIM buffer")
(defvar easypim-last-line nil "Last cursor position in PIM buffer")
(defvar easypim-mode-map nil "Key map for PIM mode")

(let ((map (make-sparse-keymap)))
  (suppress-keymap map t)
  (define-key map "\r" 'easypim-handle)
  (define-key map "b" 'easypim-back)
  (define-key map "m" 'easypim-mail)
  (define-key map "s" 'easypim-search)
  (define-key map "v" 'easypim-view)
  (setq easypim-mode-map map))

(defun easypim-prepare-string-for-shell (s)
  "Escapes the string for safe using in shell expressions"
  (when s
    (let ((r))
      (if (string-match "\'" s)
	(setq r (string-replace-match "\'" s "\'\\\'\'" t t))
	(setq r s))
      (format "\'%s\'"  r))))

(defun easypim-fill ()
  "Fills main PIM buffer with the initial view of address book"
  (setq buffer-read-only nil)
  (erase-buffer)
  (shell-command "emacs-easypim-show 2> /dev/null" "PIM")
  (setq buffer-read-only t)
  (when easypim-last-line
(goto-line easypim-last-line))
  (setq easypim-files (split-string (shell-command-to-string "emacs-easypim-files 2> /dev/null") "\n")))

(defun pim ()
  "The simple emacs tool to store personal address book"
  (interactive)
  (set-buffer (get-buffer-create "PIM"))
  (switch-to-buffer "PIM")
  (setq easypim-last-line nil)
  (easypim-fill)
  (setq major-mode 'pim-mode)
  (setq mode-name "pim-mode")
  (use-local-map easypim-mode-map))

(defun easypim-view ()
  "Shows detailed content of selected address book item"
  (interactive)
  (setq easypim-last-line (+ (count-lines 1 (point)) (if (= (current-column) 0) 1 0)))
  (let ((ln) (q))
    (setq ln easypim-last-line)
    (setq ln (- ln 1))
    (setq q easypim-files)
    (while (not (equal ln 0))
      (setq q (cdr q))
      (setq ln (- ln 1)))
    (setq q (car q))
    (when q (when (> (length q) 0)
      (setq buffer-read-only nil)
      (erase-buffer)
      (shell-command (format "cat %s 2> /dev/null" (easypim-prepare-string-for-shell q)) "PIM")
      (setq easypim-files nil)
      (setq buffer-read-only t)))))

(defun easypim-mail ()
  "Creates new message with address under the cursor in PIM buffer"
  (interactive)
  (let ((fp) (tp) (s) (tt))
    (skip-chars-forward easypim-mail-chars)
    (setq tp (point))
    (skip-chars-backward easypim-mail-chars)
    (setq fp (point))
    (setq s (buffer-substring fp tp))
    (when (> (length s) 0) (when (string-match "@" s)
      (setq tt (car (split-string (shell-command-to-string (format "emacs-easypim-find --title %s 2> /dev/null" (easypim-prepare-string-for-shell s))) "\n")))
      (if (> (length tt) 0)
        (message-mail (format "%s <%s>" tt s))
        (message-mail s))))))

(defun easypim-back ()
  "Restores the initial view of the address book"
  (interactive)
  (easypim-fill))

(defun easypim-handle ()
  "Performes the default action on the current address book item depending on its type"
  (interactive)
  (let ((ln) (q))
    (setq ln (+ (count-lines 1 (point)) (if (= (current-column) 0) 1 0)))
    (setq ln (- ln 1))
    (setq q easypim-files)
    (while (not (equal ln 0))
      (setq q (cdr q))
      (setq ln (- ln 1)))
    (setq q (car q))
    (if  (> (length q) 0)
      (easypim-view)
      (let ((fp) (lp) (p))
	(move-end-of-line nil)
	(setq lp (point))
	(move-beginning-of-line nil)
	(setq fp (point))
	(setq p (string-match "@" (buffer-substring fp lp)))
	(when p
	  (goto-char (+ fp p))
	  (easypim-mail))))))

(defun easypim-search ()
  "Searches the specified string in the content of all address book items"
  (interactive)
  (let ((s))
    (setq s (read-from-minibuffer "Search: "))
    (when (> (length s) 0)
      (setq easypim-last-line nil)
      (setq buffer-read-only nil)
      (erase-buffer)
      (shell-command (format "emacs-easypim-find %s" (easypim-prepare-string-for-shell s)) "PIM")
      (setq buffer-read-only t)
      (setq easypim-files nil))))

(provide 'easypim)
